window.Gleam.initSearch([{"doc":"clad","title":"clad","content":"# clad\n\n\n[![Package Version](https://img.shields.io/hexpm/v/clad)](https://hex.pm/packages/clad)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/clad/)\n\n\nCommand line argument decoders for Gleam.\n\nClad aims to make it as easy as possible to parse command line arguments in\nGleam. The goal is to support simple to medium complexity command line\ninterfaces while staying as minimal as possible. It is inspired by\n[minimist](https://github.com/minimistjs/minimist) and\n[gleam/json](https://hexdocs.pm/gleam_json/)\n\n\n## Usage\n\n```sh\ngleam add clad\n```\n\nThis program is in the [examples directory](https://github.com/ryanmiville/clad/tree/main/test/examples)\n\n```gleam\nimport argv\nimport clad\nimport decode/zero\n\npub type Student {\n  Student(name: String, age: Int, enrolled: Bool, classes: List(String))\n}\n\npub fn main() {\n  let decoder = {\n    use name &lt;- zero.field(&quot;name&quot;, zero.string)\n    use age &lt;- zero.field(&quot;age&quot;, zero.int)\n    use enrolled &lt;- zero.field(&quot;enrolled&quot;, zero.bool)\n    use classes &lt;- clad.positional_arguments()\n    zero.success(Student(name:, age:, enrolled:, classes:))\n  }\n\n  // args: --name Lucy --age 8 --enrolled true math science art\n  let result = clad.decode(argv.load().arguments, decoder)\n  let assert Ok(Student(&quot;Lucy&quot;, 8, True, [&quot;math&quot;, &quot;science&quot;, &quot;art&quot;])) = result\n}\n```\n\nOr, for more flexibility:\n\n```gleam\nimport argv\nimport clad\nimport decode/zero\n\npub type Student {\n  Student(name: String, age: Int, enrolled: Bool, classes: List(String))\n}\n\npub fn main() {\n  let decoder = {\n    use name &lt;- clad.opt(&quot;name&quot;, &quot;n&quot;, zero.string)\n    use age &lt;- clad.opt(&quot;age&quot;, &quot;a&quot;, zero.int)\n    use enrolled &lt;- clad.opt(&quot;enrolled&quot;, &quot;e&quot;, clad.flag())\n    use classes &lt;- clad.positional_arguments()\n    zero.success(Student(name:, age:, enrolled:, classes:))\n  }\n\n  // args: --name=Lucy -ea8 math science art\n  let result = clad.decode(argv.load().arguments, decoder)\n  let assert Ok(Student(&quot;Lucy&quot;, 8, True, [&quot;math&quot;, &quot;science&quot;, &quot;art&quot;])) = result\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/clad&gt;.\n","url":"index.html"},{"doc":"clad","title":"decode","content":"pub fn decode(\n  args: List(String),\n  decoder: Decoder(a),\n) -&gt; Result(a, List(DecodeError))\n Run a decoder on a list of command line arguments, decoding the value if it\n is of the desired type, or returning errors.\n\n This function pairs well with the [argv package](https://github.com/lpil/argv).\n\n # Examples\n \n // args: --name Lucy --email=lucy@example.com\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, dynamic.string)\n   use email &lt;- zero.field(&quot;email&quot;, dynamic.string),\n   clad.decoded(SignUp(name:, email:))\n }\n\n let result = clad.decode(argv.load().arguments, decoder)\n assert result == Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \nSynonyms:\nclad.decode\nclad decode","url":"clad.html#decode"},{"doc":"clad","title":"flag","content":"pub fn flag() -&gt; Decoder(Bool)\n A Bool decoder that returns False if value is not present\n \n let decoder = {\n   use verbose &lt;- zero.field(&quot;v&quot;, clad.flag())\n   zero.success(verbose)\n }\n let result = clad.decode([&quot;-v&quot;], decoder)\n assert result == Ok(True)\n\n let result = clad.decode([], decoder)\n assert result == Ok(False)\n \nSynonyms:\nclad.flag\nclad flag","url":"clad.html#flag"},{"doc":"clad","title":"opt","content":"pub fn opt(\n  long_name: String,\n  short_name: String,\n  field_decoder: Decoder(a),\n  next: fn(a) -&gt; Decoder(b),\n) -&gt; Decoder(b)\n Decode a command line option by either a long name or short name\n \n let decoder = {\n   use name &lt;- clad.opt(&quot;name&quot;, &quot;n&quot;, zero.string)\n   zero.success(name)\n }\n clad.decode([&quot;--name&quot;, &quot;Lucy&quot;], decoder)\n // -&gt; Ok(&quot;Lucy&quot;)\n clad.decode([&quot;-n&quot;, &quot;Lucy&quot;], decoder)\n // -&gt; Ok(&quot;Lucy&quot;)\n \nSynonyms:\nclad.opt\nclad opt","url":"clad.html#opt"},{"doc":"clad","title":"positional_arguments","content":"pub fn positional_arguments(\n  next: fn(List(String)) -&gt; Decoder(a),\n) -&gt; Decoder(a)\n Get all of the unnamed, positional arguments\n \n let decoder = {\n   use positional &lt;- clad.positional_arguments\n   zero.success(positional)\n }\n let result = clad.decode([&quot;-a1&quot;, &quot;hello&quot;, &quot;-b&quot;, &quot;2&quot;, &quot;world&quot;], decoder)\n assert result == Ok([&quot;hello&quot;, &quot;world&quot;])\n\n let result = clad.decode([&quot;-a1&quot;, &quot;-b&quot;, &quot;2&quot;], decoder)\n assert result == Ok([])\n \nSynonyms:\nclad.positional_arguments\nclad positional_arguments","url":"clad.html#positional_arguments"},{"doc":"clad","title":"clad","content":" This module encodes a list of command line arguments as a `dynamic.Dynamic` and\n provides functions to decode those arguments using a `decode/zero.Decoder`.\n\n # Encoding\n\n The following arguments:\n ```sh\n -x=3 -y 4 -n5 -abc --hello world --beep=boop foo bar baz\n ```\n will be encoded as a `dynamic.Dynamic` in this shape:\n ```json\n {\n   &quot;name&quot;: 3,\n   &quot;y&quot;: 4,\n   &quot;a&quot;: True,\n   &quot;b&quot;: True,\n   &quot;c&quot;: True,\n   &quot;hello&quot;: &quot;world&quot;,\n   &quot;beep&quot;: &quot;boop&quot;,\n   &quot;_&quot;: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;]\n }\n ```\n\n # Decoding\n\n Arguments can be decoded with a normal `zero.Decoder`\n\n ```gleam\n // args: --name Lucy --age 8 --enrolled true\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, zero.string)\n   use age &lt;- zero.field(&quot;age&quot;, zero.int)\n   use enrolled &lt;- zero.field(&quot;enrolled&quot;, zero.bool)\n   zero.success(Student(name:, age:, enrolled:))\n }\n\n let result = clad.decode(args, decoder)\n assert result == Ok(Student(&quot;Lucy&quot;, 8, True))\n ```\n Clad provides additional functions to support some common CLI behaviors.\n\n ## Boolean Flags\n\n CLI&#39;s commonly represent boolean flags just by the precense or absence of the\n option. Since Clad has no knowledge of your target record, it cannot encode\n missing flags as False.\n\n Clad provides the `flag()` decoder to handle this case.\n\n ```gleam\n // args1: --name Lucy --age 8 --enrolled\n // args2: --name Bob --age 3\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, zero.string)\n   use age &lt;- zero.field(&quot;age&quot;, zero.int)\n   use enrolled &lt;- zero.field(&quot;enrolled&quot;, clad.flag())\n   zero.success(Student(name:, age:, enrolled:))\n }\n\n let result = clad.decode(args1, decoder)\n assert result == Ok(Student(&quot;Lucy&quot;, 8, True))\n\n let result = clad.decode(args2, decoder)\n assert result == Ok(Student(&quot;Bob&quot;, 3, False))\n ```\n\n ## Alternate Names\n\n It is also common for CLI&#39;s to support long names and short names for options\n (e.g. `--name` and `-n`).\n\n Clad provides the `opt()` function for this.\n\n ```gleam\n // args1: -n Lucy -a 8 -e\n // args2: --name Bob --age 3\n\n let decoder = {\n   use name &lt;- clad.opt(long_name: &quot;name&quot;, short_name: &quot;n&quot;, zero.string)\n   use age &lt;- clad.opt(long_name: &quot;age&quot;, short_name: &quot;a&quot;, zero.int)\n   use enrolled &lt;- clad.opt(long_name: &quot;enrolled&quot;, short_name: &quot;e&quot; clad.flag())\n   zero.success(Student(name:, age:, enrolled:))\n }\n\n let result = clad.decode(args1, decoder)\n assert result == Ok(Student(&quot;Lucy&quot;, 8, True))\n\n let result = clad.decode(args2, decoder)\n assert result == Ok(Student(&quot;Bob&quot;, 3, False))\n ```\n\n ## Positional Arguments\n\n A CLI may also support positional arguments. These are any arguments that are\n not attributed to a named option. Clad provides the `positional_arguments()` decoder to\n retrieve these values.\n\n ```gleam\n // args1: -n Lucy -ea8  math science art\n // args2: --name Bob --age 3\n\n let decoder = {\n   use name &lt;- clad.opt(&quot;name&quot;, &quot;n&quot;, zero.string)\n   use age &lt;- clad.opt(&quot;age&quot;, &quot;a&quot;, zero.int)\n   use enrolled &lt;- clad.opt(&quot;enrolled&quot;, &quot;e&quot; clad.flag())\n   use classes &lt;- clad.positional_arguments()\n   zero.success(Student(name:, age:, enrolled:, classes:))\n }\n\n let result = clad.decode(args1, decoder)\n assert result == Ok(Student(&quot;Lucy&quot;, 8, True, [&quot;math&quot;, &quot;science&quot;, &quot;art&quot;]))\n\n let result = clad.decode(args2, decoder)\n assert result == Ok(Student(&quot;Bob&quot;, 3, False, []))\n ```","url":"clad.html"}]);